'use strict';

var through = require('through2');
var path = require('path');
var fs = require('fs');
var uglifyjs = require('uglify-js');
var replacepath = require('replacepath');
var rev = require('./rev');
var mod = require('./module');

var unique = function(arr){
    var newarr = [];
    var temp = {};
    arr.forEach(function(val){
        if(!temp[val]){
            temp[val] = true
            newarr.push(val)
        }
    })
    return newarr
}

module.exports = function(opts){
    var options = {
        //路径别名
        paths:{},
        //模块别名
        alias:{},
        //忽略的模块名，模块不会追加到合并的文件中
        ignore:['util', 'template', 'component'],
        //版本号长度
        verlen:7,
        //生成合并css文件
        cssdebug:false,
        //生成合并css压缩文件
        cssmin:true,
        //生成合并js文件
        jsdebug:false,
        //生成合并js压缩文件
        jsmin:true,
        //过滤资源文件路径，去除或替换一些特殊符号，返回绝对路径
        filterPath:null,
        //gulp-watch返回的对象
        watcher:null
    }

    if(opts){
        for(var i in opts){
            options[i] = opts[i]
        }
    }

    if(options.jsmin === true){
        options.jsmin = {};
    }

    if(options.jsmin.fromString !== true){
        options.jsmin.fromString = true
    }

    if(options.paths.base){
        for(var i in options.paths){
            var _path = options.paths[i];
            if(i !== 'base' && !/^[A-Z]:/.test(_path)){
                options.paths[i] = options.paths.base + _path;
            }
        }
    }

    return through.obj(function(file, enc, cb){

        if(file.isNull()){
	        return cb();
        }

        var dirname = path.dirname(file.path)+'/';
        var filecontent = file.contents.toString();
        var matchs = filecontent.match(/Nui.load\(['"][^'"]+['"]/g);

        if(matchs){
            var Module = mod(dirname, options);
            matchs.forEach(function(val, i){
                var id = matchs[i].replace(/Nui.load\(/, '').replace(/['"]/g, '').replace(/-debug|-min/g, '');
                Module.load(id, function(mod){
                    var contents = unique(mod.getContents()).join('');
                    var styles = Module.getStyles(unique(mod.loadstyles())).join('');
                    var url;
                    var filename;
                    for(var i in mod.depmodules){
                        url = mod.depmodules[i].url;
                        filename = mod.depmodules[i].filename
                        break;
                    }
                    if(fs.existsSync(url)){
                        if(styles){
                            var _path = path.join(path.dirname(url), '../style/')
                            if(!fs.existsSync(_path)){
                                fs.mkdirSync(_path)
                            }
                            if(options.cssdebug === true){
                                fs.writeFileSync(_path+filename+'-debug.css', styles);
                            }
                            if(options.cssmin === true){
                                styles = styles.replace(/\s*([\}\{\:\;\(\)\=\~\>\+\[\]\,])\s*/g, '$1')
                                               .replace(/\;\}/g, '}')
                                               .replace(/\/\*[^\/]*\*\//g, '')
                                               .replace(/\s{2,}/, ' ');
                                fs.writeFileSync(_path+filename+'-min.css', styles);
                            }
                        }

                        if(options.jsdebug === true){
                            fs.writeFileSync(url.replace(/(\.js)$/, '-debug$1'), contents);
                        }
                        if(typeof options.jsmin === 'object'){
                            contents = uglifyjs.minify(contents, options.jsmin).code;
                            fs.writeFileSync(url.replace(/(\.js)$/, '-min$1'), contents);
                        }
                    }
                })
            })
        }

        if(options.verlen > 0){
            filecontent = rev.call(Module, filecontent, dirname, options).replace(/^\s+|\s+$/g, '')
        }

        if(filecontent != ''){
            file.contents = new Buffer(filecontent);
            this.push(file);
        }
        cb();
    })
}
